import { expect, Page } from '@playwright/test'

import { appendRandomNumberToInput, copyFile, deleteFile } from './utils'

import { CREDENTIAL_TYPE_DATASTORE_UNDEFINED, CREDENTIAL_TYPES_DATA_STORE } from './data_stores/credentials_data_store'
import { CREDENTIAL_TYPES_FIELDS_LIST } from './constants/credential_fields'
import { LogInPage } from './page-objects/COMM_LoginPage'
import { ADMIN_EMAIL, ADMIN_K, ENCRYPTION_KEY, USER_K } from './constants/env_constants'
import { LeftMenuAdminPage } from './page-objects/AD_LeftMenuAdminPage'
import { LeftMenuUserPage } from './page-objects/US_LeftMenuUserPage'
import { TemplatesPage } from './page-objects/AD_TemplatesPage'
import { ViewUsersPage } from './page-objects/AD_ViewUsersPage'
import { ViewMyCredentialsPage } from './page-objects/US_ViewMyCredentialsPage'
import { UserPersonalInformationPage } from './page-objects/AD_UserPersonalInformationPage'
import { ImportTemplatePage } from './page-objects/AD_ImportTemplatePage'
import { ViewImportedDataPage } from './page-objects/AD_ViewImportedDataPage'
import { User } from './interfaces/User'
import { EncryptionKeyPage } from './page-objects/AD_EncryptionKeyPage'
import { DataProtectionPage } from './page-objects/COMM_DataProtectionPage'
import { ImportDataPage } from './page-objects/AD_ImportDataPage'
import { ENABLED_ST, ISSUED_ST, PATH_FILES_TO_IMPORT } from './constants/constants'
import { fail } from 'assert'
import { ViewCredentialPage } from './page-objects/US_ViewCredentialPage'
import { AddIdentityPage } from './page-objects/US_AddIdentityPage'
import { IdentitiesPage } from './page-objects/US_IdentitiesPage'
import { CredentialRequestPage } from './page-objects/US_CredentialRequestPage'


export async function loginAsAdmin(page: Page, url: string) {
    try {
        const loginPage = new LogInPage(page);
        const encryptionKeyPage = new EncryptionKeyPage(page);
        const dataProtectionPage = new DataProtectionPage(page);

        await loginPage.visit(url);
        await loginPage.login(ADMIN_EMAIL, ADMIN_K);

        let currentTitle = await page.title();

        if (currentTitle === 'Encryption Key – IdHub') {
            //code to set Encription Key
            await encryptionKeyPage.enterKey(ENCRYPTION_KEY);
            await encryptionKeyPage.clickSaveButton();

            await expect(page).toHaveTitle('Data protection – IdHub');
            currentTitle = await page.title();
        }

        if (currentTitle === 'Data protection – IdHub') {
            // Code to accept terms and conditions
            await dataProtectionPage.checkAcceptPrivacyCheckBox();
            await dataProtectionPage.checkAcceptCookiesCheckBox();
            await dataProtectionPage.checkAcceptLegalCheckBox();
            await dataProtectionPage.clickConfirmButton();
        }
        await expect(page).toHaveTitle('Dashboard – IdHub');

    } catch (error) {
        console.error(`Failed to login as admin: `);
        throw error;
    }
}

export async function loginAsUser(page: Page, userEmail: string, url: string) {

    try {
        const loginPage = new LogInPage(page);
        const dataProtectionPage = new DataProtectionPage(page);

        await loginPage.visit(url);
        await loginPage.login(userEmail, USER_K);

        const currentTitle = await page.title();

        if (currentTitle === 'Data Protection – IdHub') {
            // Code to accept terms and conditions
            await dataProtectionPage.checkAcceptPrivacyCheckBox();
            await dataProtectionPage.checkAcceptCookiesCheckBox();
            await dataProtectionPage.checkAcceptLegalCheckBox();
            await dataProtectionPage.clickConfirmButton();
        }
        await expect(page).toHaveTitle('Dashboard – IdHub');
    } catch (error) {
        console.error(`Failed to login as user: `);
        throw error;
    }
}

export async function clickDashboardOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const dashboardLink = await leftMenuPage.getDashboardLink();
        await dashboardLink.click({ timeout: 60000 });
        return { dashboardLink }
    } catch (error) {
        console.error(`Failed to access Dashboard from left menu: `);
        throw error;
    }
}

export async function clickUsersOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const usersLink = await leftMenuPage.getUsersLink();
        await usersLink.click()
        return { usersLink }
    } catch (error) {
        console.error(`Failed to access Users option on left menu: `);
        throw error;
    }
}
export async function clickViewUsersOnLeftMenu(page: Page) {
    try {
        clickUsersOnLeftMenu(page);
        const leftMenuPage = new LeftMenuAdminPage(page);
        const viewUsersLink = await leftMenuPage.getViewUsersLink();
        await viewUsersLink.click();
        return { viewUsersLink }
    } catch (error) {
        console.error(`Failed to access View Users Option on left menu: `);
        throw error;
    }
}
export async function clickAddUserOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const addUserLink = await leftMenuPage.getAddUserLink();
        await addUserLink.click();
        return { addUserLink }
    } catch (error) {
        console.error(`Failed to access Add Users option on left menu: `);
        throw error;
    }
}

export async function clickMyCredentialsOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuUserPage(page);
        const myCredentialsLink = await leftMenuPage.getMyCredentialsLink();
        await myCredentialsLink.click();
        return { myCredentialsLink }
    } catch (error) {
        console.error(`Failed to access My Credential option on left menu: `);
        throw error;
    }
}

export async function clickIdentitiesOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuUserPage(page);
        const identitiesLink = await leftMenuPage.getIdentitiesLink();
        await identitiesLink.click()
        return { identitiesLink }
    } catch (error) {
        console.error(`Failed to access Identities option on left menu: `);
        throw error;
    }
}

export async function clickManageRolesOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const manageRolesLink = await leftMenuPage.getManageRolesLink();
        await manageRolesLink.click()
        return { manageRolesLink }
    } catch (error) {
        console.error(`Failed to access Manage Roles option on left menu: `);
        throw error;
    }
}

export async function clickManageServicesOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const manageRolesLink = await leftMenuPage.getManageServicesLink();
        await manageRolesLink.click()
        return { manageRolesLink }
    } catch (error) {
        console.error(`Failed to access Manage Services option on left menu: `);
        throw error;
    }
}

export async function clickTemplatesOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const templatesLink = await leftMenuPage.getTemplatesLink();
        await templatesLink.click()
        return { templatesLink }
    } catch (error) {
        console.error(`Failed to access Templates option on left menu: `);
        throw error;
    }
}

export async function clickDataOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const dataLink = await leftMenuPage.getDataLink();
        await dataLink.click()
        return { dataLink }
    } catch (error) {
        console.error(`Failed access Data on left menu: `);
        throw error;
    }
}

export async function clickCredentialsOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const credentialsLink = await leftMenuPage.getCredentialsLink();
        await credentialsLink.click()
        return { credentialsLink }
    } catch (error) {
        console.error(`Failed to access Credentials On left Menu: `);
        throw error;
    }
}

export async function clickViewCredentialsOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const viewCredentialsLink = await leftMenuPage.getViewCredentialsLink();
        await viewCredentialsLink.click()
        return { viewCredentialsLink }
    } catch (error) {
        console.error(`Failed to access View Credential on left menu: `);
        throw error;
    }
}

export async function clickViewOrganizationWalletOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const viewOrganisationalWalletLink = await leftMenuPage.getOrganisationalWalletLink();
        await viewOrganisationalWalletLink.click()
        return { viewOrganisationalWalletLink }
    } catch (error) {
        console.error(`Failed to access Organisational wallet on left menu: `);
        throw error;
    }
}

export async function clickViewSchemaOnLeftMenu(page: Page, templateName: string) {
    try {
        const templatesPage = new TemplatesPage(page)
        //Access the specific template schema
        const { templatesLink } = await clickTemplatesOnLeftMenu(page)
        expect(await templatesPage.schemaIsAvailableInView(templateName)).toBeTruthy()
        await templatesPage.gotoViewSchemaPage(templateName)
    } catch (error) {
        console.error(`Failed to access the Template option on left menu : `);
        throw error;
    }
}


export async function gotoBasicInfoPageOfTheUser(page: Page, user: User) {

    const viewUsersPage = new ViewUsersPage(page)
    //Access the specific UPI
    const { viewUsersLink } = await clickViewUsersOnLeftMenu(page)
    try {
        expect(await viewUsersPage.userExists(user.email)).toBeTruthy()
    } catch (error) {
        console.error('Failed validating that user exists', error);
    }
    await viewUsersPage.gotoUserInformationPage(user)
}

export async function gotoViewCredential(page: Page, credentialType: string, status: string) {
    const myCredentialsPage = new ViewMyCredentialsPage(page)

    //Access the specific enabled credential that is expected to be in the user wallet!
    const { myCredentialsLink } = await clickMyCredentialsOnLeftMenu(page);
    try {
        expect(await myCredentialsPage.credentialExistInMyCredentials(credentialType, status)).toBeTruthy();
    } catch (error) {
        console.error('Failed accessing the enabled credential of type ${credentialType}', error);
    }
    await myCredentialsPage.gotoViewCredentialPage(credentialType, status);
}


export async function checkIfTheInformationIsValid(page: Page, user: User) {
    const userPersonalInformationPage = new UserPersonalInformationPage(page)

    try {
        expect(await userPersonalInformationPage.isTheUserInfoValid(user)).toBeTruthy();
    } catch (error) {
        console.error('Failed validating the user information:', error);
    }
}


export function expectedCredentialSubjectForUser(email: string, credentialType: string) {

    try {
        const testingDataForCredential = CREDENTIAL_TYPES_DATA_STORE[credentialType + '_data'];

        if (!testingDataForCredential) {
            console.error(`No testing data found for credential type: ${credentialType}`);
            return undefined;
        }

        let credentialSubject = testingDataForCredential.find(user => user.email.toLowerCase() === email.toLowerCase());
        return credentialSubject;
    } catch (error) {
        console.error('Failed retriving the expected credential:', error);
    }
}

export async function importSchema(page: Page, jsonSchema: string) {
    try {
        const templatesPage = new TemplatesPage(page);
        const importTemplatePage = new ImportTemplatePage(page)
        await templatesPage.gotoAddTemplatePage();
        expect(importTemplatePage.schemaIsAvailableToImport(jsonSchema)).toBeTruthy();
        await importTemplatePage.gotoImportTemplate(jsonSchema);
        return await templatesPage.schemaIsAvailableInView(jsonSchema);
    } catch (error) {
        console.error('Failed to import the schema:', error);
    }
}


export async function checkFileName(page: Page, fileName: string): Promise<string> {
    const viewImportedDataPage = new ViewImportedDataPage(page);
    let fileInTheList = await viewImportedDataPage.fileIsAvailableInView(fileName);
    if (fileInTheList) {
        let newName = appendRandomNumberToInput(fileName);
        //rename the file
        copyFile(fileName, newName);
        return newName;
    }
    return fileName;
}

export async function loadIfJsonSchemaNotAvailable(page: Page, jsonSchema: string) {
    try {
        //Ensure that the json schema associated with the xls is available
        const templatesPage = new TemplatesPage(page);
        await clickTemplatesOnLeftMenu(page);
        //check if the schema is available
        let schemaAvailable = await templatesPage.schemaIsAvailableInView(jsonSchema);
        if (!schemaAvailable) {
            //import the schema
            expect(await importSchema(page, jsonSchema)).toBeTruthy();
        }
    } catch (error) {
        console.error('Failed to load schema:', error);
    }
}

export function setCredentialValues(labels: string[], values: string[], credentialType: string): any {

    try {
        const listOfFields = CREDENTIAL_TYPES_FIELDS_LIST[credentialType];

        // Initialize an empty object to store the credential
        let credential: any = {};

        // Iterate over labels and values
        for (let i = 0; i < labels.length; i++) {
            let label = labels[i];
            let value = values[i];

            // Check if the label exists in the list of fields
            label = label.charAt(0).toLowerCase() + label.slice(1);
            if (listOfFields.includes(label)) {
                credential[label] = value;
            } else {
                // If the label does not exist, add it as "unknown_data"
                credential[label] = "unknown_data";
            }
        }
        // Return the credential object
        return credential;
    } catch (error) {
        console.error('Failed to set credential values:', error);
    }
}

export function initialize_with_undefined_values(credentialType: string) {
    return (CREDENTIAL_TYPE_DATASTORE_UNDEFINED[credentialType + '_data_undefined'])
}


/* Load of an excel file - Happy Path:
    * Check if file was loaded before. If yes, copy the original file (ie., financial-vulnerability-credential.xlxs) with a new valid with a random name
    * If the template schema (associated with the type of credential to be enabled) is not available, import it and verify the operation
    * Load the associated data file (xls) and check that appears in the list of imported data files
    * Expected behavior: the file is loaded, the message:"The file was imported successfully!" is displayed,
    * and the file appears in the imported files view. 
    */

export async function testImportDataFile_HappyPath(page: Page, fileToImport: string, jsonSchema: string, schema: string) {

    const viewImportedDataPage = new ViewImportedDataPage(page);
    const importDataPage = new ImportDataPage(page);

    await clickDataOnLeftMenu(page);

    // If a file with a FILE_TO_IMPORT was previouly loaded, rename it 
    let fileName = fileToImport;
    fileName = await checkFileName(page, fileName);

    // Check if the json schema associated with the file is loaded to be used
    // If not, load it in templates
    await loadIfJsonSchemaNotAvailable(page, jsonSchema);

    //Go to the Data option on leftmenu
    await clickDataOnLeftMenu(page);

    //Select the import button and go to the import data page
    await viewImportedDataPage.gotoImportDataPage();

    //Import excel file to enable a 'Financial vulnerability credential'for testing users
    await importDataPage.importFile(schema, PATH_FILES_TO_IMPORT + fileName, "Default");
    let actual = await viewImportedDataPage.alertFileImportedSuccessfully();

    //Rename the default excel file with the original name FILE_TO_IMPORT in the directoy PATH_FILES_TO_IMPORT
    if (fileName != fileToImport) {
        await deleteFile(fileName);
    }
    if (actual) {
        //Check if the file is in the list of imported files sucessfully
        await clickDataOnLeftMenu(page);
        await page.getByRole('link', { name: 'Date' }).click();
        await page.getByRole('link', { name: 'Date' }).click();

        expect(await viewImportedDataPage.fileIsAvailableInView(fileName)).toBeTruthy();
        expect(await viewImportedDataPage.isFileSuccessfullyLoaded(fileName)).toBeTruthy();
    } else {
        console.log("Unexpected result loading the file. Test failed.")
        fail();
    }

}

/**
    * Load of an excel file - Sad Path:
    * Check if file was loaded before. If yes, copy the original to the new one with random name.
    * If the template schema (associated with the type of credential to be enabled) is not available, import it and verify the operation
    */
export async function testImportDataFile_SadPath(page: Page, fileToImport: string, jsonSchema: string, schema: string, unsuccessAlertMessage: string) {
    const viewImportedDataPage = new ViewImportedDataPage(page);
    const importDataPage = new ImportDataPage(page);

    await clickDataOnLeftMenu(page);

    // If a file with a FILE_TO_IMPORT was previously loaded, rename it 
    let fileName = await checkFileName(page, fileToImport);

    // Check if the json schema associated with the file is loaded to be used
    // If not, load it in templates
    await loadIfJsonSchemaNotAvailable(page, jsonSchema);

    //Go to the Data option on leftmenu
    await clickDataOnLeftMenu(page);

    //Select the import button and go to the import data page
    await viewImportedDataPage.gotoImportDataPage();

    //Import excel file to enable a 'Financial vulnerability credential'. The file has no data.
    await importDataPage.importFile(schema, PATH_FILES_TO_IMPORT + fileName, "Default");
    let actual = await importDataPage.alertFileImportedUnsuccessfully(unsuccessAlertMessage);

    //Rename the default excel file with the original name FILE_TO_IMPORT in the directoy PATH_FILES_TO_IMPORT
    if (fileName != fileToImport) {
        await deleteFile(fileName);
    }
    if (actual) {
        //Check if the file is in the list of imported files as failed
        await clickDataOnLeftMenu(page);
        await page.getByRole('link', { name: 'Date' }).click();
        await page.getByRole('link', { name: 'Date' }).click();

        expect(await viewImportedDataPage.fileIsAvailableInView(fileName)).toBeTruthy();
        expect(await viewImportedDataPage.isFileSuccessfullyLoaded(fileName)).toBeFalsy();
    } else {
        console.log("Unexpected result loading an empty file. Test failed.");
        fail();
    }
}

export async function test_ViewAndCheckEnabledCredentialbyUser(page: Page, schemaType: string, user: string) {

    const enabledCredentialView = new ViewCredentialPage(page);

    // View the credential in status 'Enabled' 
    await gotoViewCredential(page, schemaType, ENABLED_ST);

    //Get the credential subject values of the  credential visualized on the screen and compare to the model 

    let metaCredential = await enabledCredentialView.buildACredentialFromInputValues(schemaType);
    let actualCredential = metaCredential.credentialSubject;
    let status = metaCredential.status;
    let expectedCredential = await expectedCredentialSubjectForUser(user, schemaType);
    expect(actualCredential).toEqual(expectedCredential);
    expect(status).toEqual(ENABLED_ST);
}

export async function test_RequestAndCheckIssuedCredentialByUser(page: Page, schemaType: string, user: string) {

    const identitiesPage = new IdentitiesPage(page);
    const addIdentityPage = new AddIdentityPage(page);
    const credentialView = new ViewCredentialPage(page);
    const credentialRequestPage = new CredentialRequestPage(page);

    await clickIdentitiesOnLeftMenu(page);

    //Add a new identity
    await identitiesPage.clickAddIdentityButton();
    let did = appendRandomNumberToInput('did');
    await addIdentityPage.addDid(did, 'Web');


    // View the Financial Vulnerabilty Credential in status 'Enabled' for the user
    await gotoViewCredential(page, schemaType, ENABLED_ST);

    // Request the credential
    await credentialView.clickRequestCredentialButton();
    await credentialRequestPage.requestCredential(did, schemaType);
    await clickMyCredentialsOnLeftMenu(page);


    await gotoViewCredential(page, schemaType, ISSUED_ST);


    //Get the credential subject values of the  credential visualized on the screen and compare to the model 
    let metaCredential = await credentialView.buildACredentialFromInputValues(schemaType);
    let actualCredential = metaCredential.credentialSubject;
    let status = metaCredential.status;
    let expectedCredential = await expectedCredentialSubjectForUser(user, schemaType);

    expect(actualCredential).toEqual(expectedCredential);
    expect(status).toEqual(ISSUED_ST);

}